@@ -1,6 +1,7 @@
 /* poppler-annot.cc: glib interface to poppler
  *
  * Copyright (C) 2007 Inigo Martinez <inigomartinez@gmail.com>
+ * Copyright (C) 2009 Carlos Garcia Campos <carlosgc@gnome.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -251,6 +252,31 @@ poppler_annot_get_contents (PopplerAnnot *poppler_annot)
   return contents ? _poppler_goo_string_to_utf8 (contents) : NULL;
 }
 
+/**
+ * poppler_annot_set_contents:
+ * @poppler_annot: a #PopplerAnnot
+ * @contents: a text string containing the new contents
+ *
+ * Sets the contents of @poppler_annot to the given value,
+ * replacing the current contents.
+ **/
+void
+poppler_annot_set_contents (PopplerAnnot *poppler_annot,
+			    const gchar  *contents)
+{
+  GooString *goo_tmp;
+  gchar *tmp;
+  gsize length = 0;
+  
+  g_return_if_fail (POPPLER_IS_ANNOT (poppler_annot));
+
+  tmp = contents ? g_convert (contents, -1, "UTF-16BE", "UTF-8", NULL, &length, NULL) : NULL;
+  goo_tmp = new GooString (tmp, length);
+  g_free (tmp);
+  poppler_annot->annot->setContents (goo_tmp);
+  delete (goo_tmp);
+}
+
 /**
  * poppler_annot_get_name:
  * @poppler_annot: a #PopplerAnnot
@@ -276,7 +302,9 @@ poppler_annot_get_name (PopplerAnnot *poppler_annot)
  * poppler_annot_get_modified:
  * @poppler_annot: a #PopplerAnnot
  *
- * Retrieves the last modification data of @poppler_annot.
+ * Retrieves the last modification data of @poppler_annot. The returned
+ * string will be either a PDF format date or a text string.
+ * See also #poppler_date_parse()
  *
  * Return value: a new allocated string with the last modification data of
  *               @poppler_annot. It must be freed with g_free() when done.
@@ -384,6 +412,26 @@ poppler_annot_markup_get_label (PopplerAnnotMarkup *poppler_annot)
   return text ? _poppler_goo_string_to_utf8 (text) : NULL;
 }
 
+/**
+ * poppler_annot_markup_has_popup:
+ * @poppler_annot: a #PopplerAnnotMarkup
+ *
+ * Return %TRUE if the markup annotation has a popup window associated
+ *
+ * Return value: %TRUE, if @poppler_annot has popup, %FALSE otherwise
+ **/
+gboolean
+poppler_annot_markup_has_popup (PopplerAnnotMarkup *poppler_annot)
+{
+  AnnotMarkup *annot;
+
+  g_return_val_if_fail (POPPLER_IS_ANNOT_MARKUP (poppler_annot), FALSE);
+
+  annot = static_cast<AnnotMarkup *>(POPPLER_ANNOT (poppler_annot)->annot);
+
+  return annot->getPopup () != NULL;
+}
+
 /**
  * poppler_annot_markup_get_popup_is_open:
  * @poppler_annot: a #PopplerAnnotMarkup
@@ -409,6 +457,41 @@ poppler_annot_markup_get_popup_is_open (PopplerAnnotMarkup *poppler_annot)
   return FALSE;
 }
 
+/**
+ * poppler_annot_markup_get_popup_rectangle:
+ * @poppler_annot: a #PopplerAnnotMarkup
+ * @poppler_rect: a #PopplerRectangle to store the popup rectangle
+ *
+ * Retrieves the rectangle of the popup annot related to @poppler_annot.
+ *
+ * Return value: %TRUE if #PopplerRectangle was correctly filled,
+ *               %FALSE otherwise
+ **/
+gboolean
+poppler_annot_markup_get_popup_rectangle (PopplerAnnotMarkup *poppler_annot,
+					  PopplerRectangle   *poppler_rect)
+{
+  AnnotMarkup *annot;
+  Annot *annot_popup;
+  PDFRectangle *annot_rect;
+
+  g_return_val_if_fail (POPPLER_IS_ANNOT_MARKUP (poppler_annot), FALSE);
+  g_return_val_if_fail (poppler_rect != NULL, FALSE);
+
+  annot = static_cast<AnnotMarkup *>(POPPLER_ANNOT (poppler_annot)->annot);
+  annot_popup = annot->getPopup ();
+  if (!annot_popup)
+    return FALSE;
+
+  annot_rect = annot_popup->getRect ();
+  poppler_rect->x1 = annot_rect->x1;
+  poppler_rect->x2 = annot_rect->x2;
+  poppler_rect->y1 = annot_rect->y1;
+  poppler_rect->y2 = annot_rect->y2;
+
+  return TRUE;
+}
+
 /**
 * poppler_annot_markup_get_opacity:
 * @poppler_annot: a #PopplerAnnotMarkup
@@ -434,22 +517,22 @@ poppler_annot_markup_get_date (PopplerAnnotMarkup *poppler_annot)
 {
   AnnotMarkup *annot;
   GooString *annot_date;
+  time_t timet;
 
   g_return_val_if_fail (POPPLER_IS_ANNOT_MARKUP (poppler_annot), NULL);
 
   annot = static_cast<AnnotMarkup *>(POPPLER_ANNOT (poppler_annot)->annot);
+  annot_date = annot->getDate ();
+  if (!annot_date)
+    return NULL;
 
-  if ((annot_date = annot->getDate ())) {
-    GDateYear year = g_ascii_digit_value (annot_date->getChar(2)) * 1000 +
-                     g_ascii_digit_value (annot_date->getChar(3)) * 100 +
-                     g_ascii_digit_value (annot_date->getChar(4)) * 10 +
-                     g_ascii_digit_value (annot_date->getChar(5));
-    GDateMonth month = (GDateMonth) (g_ascii_digit_value (annot_date->getChar(6)) * 10 +
-                                     g_ascii_digit_value (annot_date->getChar(7)));
-    GDateDay day = g_ascii_digit_value (annot_date->getChar(8)) * 10 +
-                   g_ascii_digit_value (annot_date->getChar(9));
+  if (_poppler_convert_pdf_date_to_gtime (annot_date, &timet)) {
+    GDate *date;
 
-    return g_date_new_dmy (day, month, year);
+    date = g_date_new ();
+    g_date_set_time_t (date, timet);
+
+    return date;
   }
 
   return NULL;
@@ -703,18 +786,9 @@ poppler_annot_free_text_get_callout_line (PopplerAnnotFreeText *poppler_annot)
 }
 
 /* PopplerAnnotCalloutLine */
-GType
-poppler_annot_callout_line_get_type (void)
-{
-  static GType our_type = 0;
-
-  if (our_type == 0)
-    our_type = g_boxed_type_register_static ("PopplerAnnotCalloutLine",
-                                             (GBoxedCopyFunc) poppler_annot_callout_line_copy,
-                                             (GBoxedFreeFunc) poppler_annot_callout_line_free);
-
-  return our_type;
-}
+POPPLER_DEFINE_BOXED_TYPE (PopplerAnnotCalloutLine, poppler_annot_callout_line,
+			   poppler_annot_callout_line_copy,
+			   poppler_annot_callout_line_free)
 
 /**
  * poppler_annot_callout_line_new:
