@@ -16,6 +16,7 @@
 // Copyright (C) 2008 Julien Rebetez <julien@fhtagn.net>
 // Copyright (C) 2008 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2009 Matthias Franz <matthias@ktug.or.kr>
+// Copyright (C) 2009 David Benjamin <davidben@mit.edu>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -237,6 +238,7 @@ DecryptStream::~DecryptStream() {
 void DecryptStream::reset() {
   int i;
 
+  charactersRead = 0;
   str->reset();
   switch (algo) {
   case cryptRC4:
@@ -254,6 +256,10 @@ void DecryptStream::reset() {
   }
 }
 
+int DecryptStream::getPos() {
+  return charactersRead;
+}
+
 int DecryptStream::getChar() {
   Guchar in[16];
   int c, i;
@@ -288,6 +294,8 @@ int DecryptStream::getChar() {
     }
     break;
   }
+  if (c != EOF)
+    charactersRead++;
   return c;
 }
 
