@@ -6,6 +6,20 @@
 //
 //========================================================================
 
+//========================================================================
+//
+// Modified under the Poppler project - http://poppler.freedesktop.org
+//
+// All changes made under the Poppler project to this file are licensed
+// under GPL version 2 or later
+//
+// Copyright (C) 2009 Albert Astals Cid <aacid@kde.org>
+//
+// To see a description of the changes please see the Changelog file that
+// came with your tarball or type make ChangeLog if you are building from git
+//
+//========================================================================
+
 #ifndef FUNCTION_H
 #define FUNCTION_H
 
@@ -20,6 +34,7 @@ class Dict;
 class Stream;
 struct PSObject;
 class PSStack;
+class PopplerCache;
 
 //------------------------------------------------------------------------
 // Function
@@ -220,8 +235,10 @@ private:
 
   GooString *codeString;
   PSObject *code;
+  PSStack *stack;
   int codeSize;
   GBool ok;
+  PopplerCache *cache;
 };
 
 #endif
