@@ -13,6 +13,7 @@
 //
 // Copyright (C) 2006 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2008 Tomas Are Haavet <tomasare@gmail.com>
+// Copyright (C) 2009 Thomas Freitag <Thomas.Freitag@alfa.de>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -90,6 +91,15 @@ static inline Guchar splashCMYK8Y(SplashColorPtr cmyk8) { return cmyk8[2]; }
 static inline Guchar splashCMYK8K(SplashColorPtr cmyk8) { return cmyk8[3]; }
 #endif
 
+static inline void splashClearColor(SplashColorPtr dest) {
+  dest[0] = 0;
+  dest[1] = 0;
+  dest[2] = 0;
+#if SPLASH_CMYK
+  dest[3] = 0;
+#endif
+}
+
 static inline void splashColorCopy(SplashColorPtr dest, SplashColorPtr src) {
   dest[0] = src[0];
   dest[1] = src[1];
